import uuid
from typing import Any, List, Optional

import strawberry
import strawberry_django
from strawberry import auto
from strawberry.types import Info
from strawberry_django.mutations import mutations
from strawberry_django.pagination import OffsetPaginationInput

from api.models import Sector
from api.types.type_sector import SectorFilter, SectorOrder, TypeSector
from api.utils.enums import DatasetStatus


@strawberry.input
class SectorInput:
    name: str
    description: Optional[str] = None
    parent_id: Optional[uuid.UUID] = None


@strawberry_django.partial(Sector)
class SectorInputPartial:
    id: uuid.UUID
    name: Optional[str] = None
    description: Optional[str] = None
    slug: Optional[str] = None
    parent_id: Optional[uuid.UUID] = None


@strawberry.type(name="Query")
class Query:
    sectors: list[TypeSector] = strawberry_django.field()

    @strawberry_django.field
    def sector(self, info: Info, id: uuid.UUID) -> Optional[TypeSector]:
        """Get sector by ID."""
        try:
            sector = Sector.objects.get(id=id)
            return TypeSector.from_django(sector)
        except Sector.DoesNotExist:
            raise ValueError(f"Sector with ID {id} does not exist.")

    @strawberry_django.field(
        filters=SectorFilter,
        pagination=True,
        order=SectorOrder,
    )
    def active_sectors(
        self,
        info: Info,
        filters: Optional[SectorFilter] = strawberry.UNSET,
        pagination: Optional[OffsetPaginationInput] = strawberry.UNSET,
        order: Optional[SectorOrder] = strawberry.UNSET,
    ) -> list[TypeSector]:
        """Get sectors with published datasets."""
        # Start with base queryset filtering for active sectors
        queryset = Sector.objects.filter(
            datasets__status=DatasetStatus.PUBLISHED
        ).distinct()

        # Apply filters if provided
        if filters is not strawberry.UNSET:
            queryset = strawberry_django.filters.apply(filters, queryset, info)

        # Apply ordering if provided
        if order is not strawberry.UNSET:
            queryset = strawberry_django.ordering.apply(order, queryset, info)

        # Apply pagination if provided
        if pagination is not strawberry.UNSET:
            # Apply pagination to the list
            queryset = strawberry_django.pagination.apply(pagination, queryset)

        return [TypeSector.from_django(instance) for instance in queryset]


@strawberry.type
class Mutation:
    @strawberry_django.mutation(handle_django_errors=True)
    def create_sector(self, info: Info, input: SectorInput) -> TypeSector:
        """Create a new sector."""
        # Create a new sector with the provided name and description
        sector = Sector(
            name=input.name,
            description=input.description,
        )

        # Handle parent_id if provided
        if input.parent_id is not None:
            try:
                parent_sector = Sector.objects.get(id=input.parent_id)
                sector.parent_id = parent_sector
            except Sector.DoesNotExist:
                raise ValueError(
                    f"Parent sector with ID {input.parent_id} does not exist."
                )

        # Save the sector to generate the slug
        sector.save()

        return TypeSector.from_django(sector)

    @strawberry_django.mutation(handle_django_errors=True)
    def update_sector(
        self, info: Info, input: SectorInputPartial
    ) -> Optional[TypeSector]:
        """Update an existing sector."""
        try:
            sector = Sector.objects.get(id=input.id)

            # Get the fields to update, excluding id and parent_id (handle separately)
            input_dict = {
                f.name: getattr(input, f.name)
                for f in Sector._meta.fields
                if hasattr(input, f.name) and f.name not in ["id", "parent_id"]
            }

            # Update the sector with the provided fields
            for field, value in input_dict.items():
                if value is not None:
                    setattr(sector, field, value)

            # Handle parent_id separately if it's provided
            if hasattr(input, "parent_id") and input.parent_id is not None:
                try:
                    parent_sector = Sector.objects.get(id=input.parent_id)
                    sector.parent_id = parent_sector
                except Sector.DoesNotExist:
                    raise ValueError(
                        f"Parent sector with ID {input.parent_id} does not exist."
                    )

            sector.save()

            return TypeSector.from_django(sector)
        except Sector.DoesNotExist:
            raise ValueError(f"Sector with ID {input.id} does not exist.")

    @strawberry_django.mutation(handle_django_errors=False)
    def delete_sector(self, info: Info, sector_id: uuid.UUID) -> bool:
        """Delete a sector."""
        try:
            sector = Sector.objects.get(id=sector_id)
            sector.delete()
            return True
        except Sector.DoesNotExist:
            raise ValueError(f"Sector with ID {sector_id} does not exist.")



"""UseCase resource client for DataSpace SDK."""

from typing import Any, Dict, List, Optional

from dataspace_sdk.base import BaseAPIClient


class UseCaseClient(BaseAPIClient):
    """Client for interacting with UseCase resources."""

    def search(
        self,
        query: Optional[str] = None,
        tags: Optional[List[str]] = None,
        sectors: Optional[List[str]] = None,
        geographies: Optional[List[str]] = None,
        status: Optional[str] = None,
        running_status: Optional[str] = None,
        sort: Optional[str] = None,
        page: int = 1,
        page_size: int = 10,
    ) -> Dict[str, Any]:
        """
        Search for use cases using Elasticsearch.

        Args:
            query: Search query string
            tags: Filter by tags
            sectors: Filter by sectors
            geographies: Filter by geographies
            status: Filter by status (DRAFT, PUBLISHED, etc.)
            running_status: Filter by running status (INITIATED, ONGOING, COMPLETED)
            sort: Sort order (recent, alphabetical, started_on, completed_on)
            page: Page number (1-indexed)
            page_size: Number of results per page

        Returns:
            Dictionary containing search results and metadata
        """
        params: Dict[str, Any] = {
            "page": page,
            "page_size": page_size,
        }

        if query:
            params["q"] = query
        if tags:
            params["tags"] = ",".join(tags)
        if sectors:
            params["sectors"] = ",".join(sectors)
        if geographies:
            params["geographies"] = ",".join(geographies)
        if status:
            params["status"] = status
        if running_status:
            params["running_status"] = running_status
        if sort:
            params["sort"] = sort

        return self.get("/api/search/usecase/", params=params)

    def get_by_id(self, usecase_id: int) -> Dict[str, Any]:
        """
        Get a use case by ID using GraphQL.

        Args:
            usecase_id: ID of the use case

        Returns:
            Dictionary containing use case information
        """
        query = """
        query GetUseCase($id: ID!) {
            useCase(id: $id) {
                id
                title
                summary
                status
                runningStatus
                platformUrl
                logo
                startedOn
                completedOn
                createdAt
                updatedAt
                tags {
                    id
                    value
                }
                sectors {
                    id
                    name
                }
                geographies {
                    id
                    name
                }
                sdgs {
                    id
                    name
                    description
                }
                datasets {
                    id
                    title
                    description
                }
                organizations {
                    id
                    organization {
                        id
                        name
                    }
                    relationshipType
                }
                contributors {
                    id
                    username
                    firstName
                    lastName
                }
            }
        }
        """

        response = self.post(
            "/api/graphql",
            json_data={
                "query": query,
                "variables": {"id": str(usecase_id)},
            },
        )

        if "errors" in response:
            from dataspace_sdk.exceptions import DataSpaceAPIError

            raise DataSpaceAPIError(f"GraphQL error: {response['errors']}")

        result: Dict[str, Any] = response.get("data", {}).get("useCase", {})
        return result

    def list_all(
        self,
        status: Optional[str] = None,
        running_status: Optional[str] = None,
        organization_id: Optional[str] = None,
        limit: int = 10,
        offset: int = 0,
    ) -> Any:
        """
        List all use cases with pagination using GraphQL.

        Args:
            status: Filter by status
            running_status: Filter by running status
            organization_id: Filter by organization
            limit: Number of results to return
            offset: Number of results to skip

        Returns:
            Dictionary containing list of use cases
        """
        query = """
        query ListUseCases($filters: UseCaseFilter, $pagination: OffsetPaginationInput) {
            useCases(filters: $filters, pagination: $pagination) {
                id
                title
                summary
                status
                runningStatus
                platformUrl
                startedOn
                completedOn
                createdAt
                updatedAt
                tags {
                    id
                    value
                }
                sectors {
                    id
                    name
                }
                organizations {
                    id
                    organization {
                        id
                        name
                    }
                    relationshipType
                }
            }
        }
        """

        filters: Dict[str, Any] = {}
        if status:
            filters["status"] = status
        if running_status:
            filters["runningStatus"] = running_status
        if organization_id:
            filters["organizations"] = {"organization": {"id": {"exact": organization_id}}}

        variables: Dict[str, Any] = {
            "pagination": {"limit": limit, "offset": offset},
        }
        if filters:
            variables["filters"] = filters

        response = self.post(
            "/api/graphql",
            json_data={
                "query": query,
                "variables": variables,
            },
        )

        if "errors" in response:
            from dataspace_sdk.exceptions import DataSpaceAPIError

            raise DataSpaceAPIError(f"GraphQL error: {response['errors']}")

        data = response.get("data", {})
        usecases_result: Any = data.get("useCases", []) if isinstance(data, dict) else []
        return usecases_result

    def get_organization_usecases(
        self,
        organization_id: str,
        limit: int = 10,
        offset: int = 0,
    ) -> Any:
        """
        Get use cases for a specific organization.

        Args:
            organization_id: UUID of the organization
            limit: Number of results to return
            offset: Number of results to skip

        Returns:
            Dictionary containing organization's use cases
        """
        return self.list_all(
            organization_id=organization_id,
            limit=limit,
            offset=offset,
        )



from rest_framework import status, views
from rest_framework.permissions import AllowAny
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework_simplejwt.tokens import RefreshToken

from api.utils.keycloak_utils import keycloak_manager


class KeycloakLoginView(views.APIView):
    """
    View for handling Keycloak login and token exchange.
    Accepts Keycloak tokens and creates Django tokens.
    """

    permission_classes = [AllowAny]

    def post(self, request: Request) -> Response:
        # Get the Keycloak token from the request
        keycloak_token = request.data.get("token")
        if not keycloak_token:
            return Response(
                {"error": "Keycloak token is required"},
                status=status.HTTP_400_BAD_REQUEST,
            )

        # Validate the token and get user info
        user_info = keycloak_manager.validate_token(keycloak_token)
        if not user_info:
            return Response(
                {"error": "Invalid or expired token"},
                status=status.HTTP_401_UNAUTHORIZED,
            )

        # Get user roles and organizations from the token
        roles = keycloak_manager.get_user_roles(keycloak_token)
        organizations = keycloak_manager.get_user_organizations(keycloak_token)

        # Sync the user information with our database
        user = keycloak_manager.sync_user_from_keycloak(user_info, roles, organizations)
        if not user:
            return Response(
                {"error": "Failed to synchronize user information"},
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
            )

        # Create Django tokens for the user
        refresh = RefreshToken.for_user(user)

        return Response(
            {
                "refresh": str(refresh),
                "access": str(refresh.access_token),
                "user": {
                    "id": user.id,
                    "username": user.username,
                    "email": user.email,
                    "first_name": user.first_name,
                    "last_name": user.last_name,
                    "is_staff": user.is_staff,
                    "is_superuser": user.is_superuser,
                    "organizations": [
                        {
                            "id": org.organization.id,  # type: ignore[attr-defined]
                            "name": org.organization.name,  # type: ignore[attr-defined]
                            "role": org.role.name,  # type: ignore[attr-defined]
                        }
                        for org in user.organizationmembership_set.all()  # type: ignore[union-attr, arg-type]
                    ],
                },
            }
        )


class UserInfoView(views.APIView):
    """
    View for getting the current user's information.
    """

    def get(self, request: Request) -> Response:
        user = request.user
        return Response(
            {
                "id": user.id,
                "username": user.username,
                "email": user.email,  # type: ignore[union-attr]
                "first_name": user.first_name,  # type: ignore[union-attr]
                "last_name": user.last_name,  # type: ignore[union-attr]
                "is_staff": user.is_staff,
                "is_superuser": user.is_superuser,
                "organizations": [
                    {
                        "id": org.organization.id,  # type: ignore[attr-defined]
                        "name": org.organization.name,  # type: ignore[attr-defined]
                        "role": org.role.name,  # type: ignore[attr-defined]
                    }
                    for org in user.organizationmembership_set.all()  # type: ignore[union-attr, arg-type]
                ],
            }
        )



from typing import Any

from django.db import models
from django.utils.text import slugify

from api.utils.enums import OrganizationTypes
from api.utils.file_paths import _organization_directory_path


class Organization(models.Model):
    name = models.CharField(max_length=200)
    description = models.CharField(max_length=1000)
    logo = models.ImageField(
        upload_to=_organization_directory_path, blank=True, null=True, max_length=300
    )
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)
    homepage = models.URLField(blank=True)
    contact_email = models.EmailField(blank=True, null=True)
    organization_types = models.CharField(
        max_length=100, choices=OrganizationTypes.choices
    )
    parent = models.ForeignKey(
        "api.Organization",
        unique=False,
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name="parent_field",
    )
    slug = models.SlugField(max_length=75, null=True, blank=False, unique=True)
    github_profile = models.URLField(blank=True, null=True)
    linkedin_profile = models.URLField(blank=True, null=True)
    twitter_profile = models.URLField(blank=True, null=True)
    location = models.CharField(max_length=255, blank=True, null=True)

    def save(self, *args: Any, **kwargs: Any) -> None:
        self.slug = slugify(self.name)
        super().save(*args, **kwargs)

    def __str__(self):
        return self.name

    class Meta:
        db_table = "organization"



"""Example: Advanced search with filters and pagination."""

from dataspace_sdk import DataSpaceClient

# Initialize and login
client = DataSpaceClient(base_url="https://api.dataspace.example.com")
keycloak_token = "your_keycloak_token_here"

try:
    client.login(keycloak_token=keycloak_token)

    # Advanced dataset search with multiple filters
    print("=== Advanced Dataset Search ===\n")
    datasets = client.datasets.search(
        query="health data",
        tags=["public-health", "covid-19"],
        sectors=["Health", "Social Welfare"],
        geographies=["India", "Karnataka"],
        status="PUBLISHED",
        access_type="OPEN",
        sort="recent",
        page=1,
        page_size=10,
    )

    print(f"Total results: {datasets['total']}")
    print(f"Page: {datasets['page']} of {datasets['total_pages']}")
    print("\nResults:")
    for dataset in datasets["results"]:
        print(f"\n- Title: {dataset['title']}")
        print(f"  Status: {dataset['status']}")
        print(f"  Access: {dataset['access_type']}")
        print(f"  Tags: {', '.join([t['value'] for t in dataset.get('tags', [])])}")

    # Advanced AI model search
    print("\n\n=== Advanced AI Model Search ===\n")
    models = client.aimodels.search(
        query="language model",
        tags=["nlp", "text-generation"],
        model_type="LLM",
        provider="OPENAI",
        status="ACTIVE",
        sort="recent",
        page=1,
        page_size=10,
    )

    print(f"Total results: {models['total']}")
    print("\nResults:")
    for model in models["results"]:
        print(f"\n- Name: {model.get('displayName', model.get('name'))}")
        print(f"  Type: {model.get('modelType')}")
        print(f"  Provider: {model.get('provider')}")
        print(f"  Status: {model.get('status')}")

    # Advanced use case search
    print("\n\n=== Advanced Use Case Search ===\n")
    usecases = client.usecases.search(
        query="health monitoring",
        tags=["health", "monitoring", "real-time"],
        sectors=["Health"],
        status="PUBLISHED",
        running_status="COMPLETED",
        sort="completed_on",
        page=1,
        page_size=10,
    )

    print(f"Total results: {usecases['total']}")
    print("\nResults:")
    for usecase in usecases["results"]:
        print(f"\n- Title: {usecase['title']}")
        print(f"  Status: {usecase['status']}")
        print(f"  Running Status: {usecase.get('running_status')}")
        print(f"  Started: {usecase.get('started_on')}")
        print(f"  Completed: {usecase.get('completed_on')}")

    # Pagination example - fetch all results
    print("\n\n=== Pagination Example ===\n")
    all_datasets = []
    page = 1
    page_size = 20

    while True:
        results = client.datasets.search(query="education", page=page, page_size=page_size)

        all_datasets.extend(results["results"])
        print(f"Fetched page {page}: {len(results['results'])} results")

        if len(results["results"]) < page_size:
            break

        page += 1

    print(f"\nTotal datasets fetched: {len(all_datasets)}")

except Exception as e:
    print(f"Error: {e}")



# mypy: disable-error-code=attr-defined
from django.urls import path
from rest_framework_simplejwt.views import TokenRefreshView

from authorization.views import KeycloakLoginView, UserConsentView, UserInfoView

urlpatterns = [
    # Authentication endpoints
    path("keycloak/login/", KeycloakLoginView.as_view(), name="keycloak_login"),
    path("token/refresh/", TokenRefreshView.as_view(), name="token_refresh"),
    path("user/info/", UserInfoView.as_view(), name="user_info"),
    # User consent endpoints
    path("user/consent/", UserConsentView.as_view(), name="user_consent"),
]



from typing import Any, Dict, List, Optional, Type, TypeVar, Union, cast

from django.db import models
from django.db.models import Field, Model, QuerySet

T = TypeVar("T", bound=models.Model)
GraphQLType = TypeVar("GraphQLType")


def get_graphql_type_fields_name(type_: Type[GraphQLType]) -> Dict[str, Any]:
    """Get field names from a GraphQL type."""
    fields = type_.__dict__.get("__dataclass_fields__", {})
    return cast(Dict[str, Any], fields.keys())


def convert_to_graphql_type(
    db_model_object: Model, graphql_type: Type[GraphQLType]
) -> GraphQLType:
    """Convert a Django model object to a GraphQL type."""
    fields = get_graphql_type_fields_name(graphql_type)
    field_values = {
        field: getattr(db_model_object, field)
        for field in fields
        if hasattr(db_model_object, field)
    }
    return cast(GraphQLType, graphql_type(**field_values))



"""Resource clients for DataSpace SDK."""

from dataspace_sdk.resources.aimodels import AIModelClient
from dataspace_sdk.resources.datasets import DatasetClient
from dataspace_sdk.resources.usecases import UseCaseClient

__all__ = ["DatasetClient", "AIModelClient", "UseCaseClient"]



from django.contrib.auth.models import AbstractUser
from django.db import models
from django.utils.text import slugify

from api.utils.file_paths import _user_profile_image_directory_path
from authorization.consent import UserConsent


class Role(models.Model):
    """
    Role model for defining permissions in the system.
    """

    name = models.CharField(max_length=50, unique=True)
    description = models.TextField(blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    # Define permissions for each role
    can_view = models.BooleanField(default=True)
    can_add = models.BooleanField(default=False)
    can_change = models.BooleanField(default=False)
    can_delete = models.BooleanField(default=False)

    def __str__(self) -> str:
        return self.name

    class Meta:
        db_table = "role"


class User(AbstractUser):
    """
    Custom User model that extends Django's AbstractUser.
    This model adds organization-related fields for Keycloak integration.
    """

    # Keycloak ID field to store the Keycloak user ID
    keycloak_id = models.CharField(max_length=255, unique=True, null=True, blank=True)

    # Organization relationship - a user can belong to multiple organizations
    organizations: models.ManyToManyField = models.ManyToManyField(
        "api.Organization", through="OrganizationMembership", related_name="members"
    )

    # Additional user profile fields
    bio = models.TextField(blank=True, null=True)
    profile_picture = models.ImageField(
        upload_to=_user_profile_image_directory_path, blank=True, null=True
    )
    github_profile = models.URLField(blank=True, null=True)
    linkedin_profile = models.URLField(blank=True, null=True)
    twitter_profile = models.URLField(blank=True, null=True)
    location = models.CharField(max_length=255, blank=True, null=True)

    @property
    def full_name(self) -> str:
        """Return the user's full name."""
        return f"{self.first_name} {self.last_name}"

    class Meta:
        db_table = "ds_user"


class OrganizationMembership(models.Model):
    """
    Intermediate model for User-Organization relationship.
    This model stores the role of a user within an organization.
    """

    user = models.ForeignKey("authorization.User", on_delete=models.CASCADE)
    organization = models.ForeignKey("api.Organization", on_delete=models.CASCADE)
    role = models.ForeignKey(Role, on_delete=models.CASCADE)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        db_table = "organization_membership"
        unique_together = ("user", "organization")


class DatasetPermission(models.Model):
    """
    Model for dataset-specific permissions.
    This allows for more granular control beyond organization membership.
    """

    user = models.ForeignKey("authorization.User", on_delete=models.CASCADE)
    dataset = models.ForeignKey("api.Dataset", on_delete=models.CASCADE)
    role = models.ForeignKey(Role, on_delete=models.CASCADE)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        db_table = "dataset_permission"
        unique_together = ("user", "dataset")



# from typing import Any, Dict, List, Optional, Type, TypeVar, cast

# # type: ignore[import-untyped]
# from django_elasticsearch_dsl_drf.constants import (
#     LOOKUP_FILTER_TERMS,
#     LOOKUP_FILTER_RANGE,
#     LOOKUP_FILTER_PREFIX,
#     LOOKUP_FILTER_WILDCARD,
#     LOOKUP_QUERY_IN,
#     LOOKUP_QUERY_GT,
#     LOOKUP_QUERY_GTE,
#     LOOKUP_QUERY_LT,
#     LOOKUP_QUERY_LTE,
#     LOOKUP_QUERY_EXCLUDE,
# )
# # type: ignore[import-untyped]
# from django_elasticsearch_dsl_drf.filter_backends import (
#     FilteringFilterBackend,
#     IdsFilterBackend,
#     OrderingFilterBackend,
#     DefaultOrderingFilterBackend,
#     SearchFilterBackend,
#     CompoundSearchFilterBackend,
#     MultiMatchSearchFilterBackend,
#     FacetedSearchFilterBackend,
# )
# # type: ignore[import-untyped]
# from django_elasticsearch_dsl_drf.viewsets import BaseDocumentViewSet, DocumentViewSet
# # type: ignore[import-untyped]
# from django_elasticsearch_dsl_drf.pagination import PageNumberPagination

# from api.views.search_dataset import DatasetDocumentSerializer
# from search.documents import DatasetDocument


# class DatasetDocumentView(BaseDocumentViewSet):
#     """
#     The Dataset Document view.

#     This view provides a way to interact with the DatasetDocument.
#     """

#     document: Type[DatasetDocument] = DatasetDocument
#     serializer_class: Type[DatasetDocumentSerializer] = DatasetDocumentSerializer
#     pagination_class: Type[PageNumberPagination] = PageNumberPagination
#     lookup_field: str = 'id'
#     filter_backends: List[Any] = [
#         FilteringFilterBackend,
#         IdsFilterBackend,
#         OrderingFilterBackend,
#         DefaultOrderingFilterBackend,
#         SearchFilterBackend,
#         FacetedSearchFilterBackend
#     ]

#     # Define search fields
#     search_fields: tuple = (
#         'metadata.value',
#         'title',
#         'description',
#         'tags'
#     )

#     # Define filter fields
#     filter_fields: Dict[str, Dict[str, Any]] = {
#         'id': {
#             'field': 'id',
#             'lookups': [
#                 LOOKUP_QUERY_IN,
#             ],
#         },
#         'title': {
#             'field': 'title',
#             'lookups': [
#                 LOOKUP_FILTER_TERMS,
#                 LOOKUP_FILTER_PREFIX,
#                 LOOKUP_FILTER_WILDCARD,
#                 LOOKUP_QUERY_IN,
#                 LOOKUP_QUERY_EXCLUDE,
#             ],
#         },
#         'title.raw': {
#             'field': 'title.raw',
#             'lookups': [
#                 LOOKUP_FILTER_TERMS,
#                 LOOKUP_FILTER_PREFIX,
#                 LOOKUP_FILTER_WILDCARD,
#                 LOOKUP_QUERY_IN,
#                 LOOKUP_QUERY_EXCLUDE,
#             ],
#         },
#         'metadata': {
#             'field': 'metadata.value',
#             'lookups': [
#                 LOOKUP_FILTER_TERMS,
#                 LOOKUP_FILTER_PREFIX,
#                 LOOKUP_FILTER_WILDCARD,
#             ]
#         },
#         'tags': {
#             'field': 'tags',
#             'lookups': [
#                 LOOKUP_FILTER_TERMS,
#                 LOOKUP_FILTER_PREFIX,
#                 LOOKUP_FILTER_WILDCARD,
#                 LOOKUP_QUERY_IN,
#                 LOOKUP_QUERY_EXCLUDE,
#             ],
#         },
#         'tags.raw': {
#             'field': 'tags.raw',
#             'lookups': [
#                 LOOKUP_FILTER_TERMS,
#                 LOOKUP_FILTER_PREFIX,
#                 LOOKUP_FILTER_WILDCARD,
#                 LOOKUP_QUERY_IN,
#                 LOOKUP_QUERY_EXCLUDE,
#             ],
#         },
#     }

#     ordering_fields: Dict[str, str] = {
#         'id': 'id',
#         'title': 'title.raw',
#     }

#     ordering: tuple = ('_score', 'id',)


# class DatasetCompoundSearchBackendDocumentViewSet(BaseDocumentViewSet):
#     """
#     The Dataset Document view with compound search.

#     This view provides a way to interact with the DatasetDocument using compound search.
#     """

#     document: Type[DatasetDocument] = DatasetDocument
#     serializer_class: Type[DatasetDocumentSerializer] = DatasetDocumentSerializer
#     lookup_field: str = 'id'
#     multi_match_options: Dict[str, str] = {
#         'type': 'best_fields'
#     }
#     filter_backends: List[Any] = [
#         FilteringFilterBackend,
#         OrderingFilterBackend,
#         DefaultOrderingFilterBackend,
#         CompoundSearchFilterBackend,
#     ]

#     filter_fields: Dict[str, Dict[str, Any]] = {
#         'id': {
#             'field': 'id',
#             'lookups': [
#                 LOOKUP_QUERY_IN,
#             ],
#         },
#         'title': {
#             'field': 'title',
#             'lookups': [
#                 LOOKUP_FILTER_TERMS,
#                 LOOKUP_FILTER_PREFIX,
#                 LOOKUP_FILTER_WILDCARD,
#                 LOOKUP_QUERY_IN,
#                 LOOKUP_QUERY_EXCLUDE,
#             ],
#         },
#         'title.raw': {
#             'field': 'title.raw',
#             'lookups': [
#                 LOOKUP_FILTER_TERMS,
#                 LOOKUP_FILTER_PREFIX,
#                 LOOKUP_FILTER_WILDCARD,
#                 LOOKUP_QUERY_IN,
#                 LOOKUP_QUERY_EXCLUDE,
#             ],
#         },
#         'metadata': {
#             'field': 'metadata.value',
#             'lookups': [
#                 LOOKUP_FILTER_TERMS,
#                 LOOKUP_FILTER_PREFIX,
#                 LOOKUP_FILTER_WILDCARD,
#             ]
#         },
#         'tags': {
#             'field': 'tags',
#             'lookups': [
#                 LOOKUP_FILTER_TERMS,
#                 LOOKUP_FILTER_PREFIX,
#                 LOOKUP_FILTER_WILDCARD,
#                 LOOKUP_QUERY_IN,
#                 LOOKUP_QUERY_EXCLUDE,
#             ],
#         },
#         'tags.raw': {
#             'field': 'tags.raw',
#             'lookups': [
#                 LOOKUP_FILTER_TERMS,
#                 LOOKUP_FILTER_PREFIX,
#                 LOOKUP_FILTER_WILDCARD,
#                 LOOKUP_QUERY_IN,
#                 LOOKUP_QUERY_EXCLUDE,
#             ],
#         },
#     }

#     search_fields: Dict[str, Optional[Dict[str, str]]] = {
#         'title': {'fuzziness': 'AUTO'},
#         'description': {'fuzziness': 'AUTO'},
#         'tags': None,
#         'metadata.value': None
#     }

#     ordering_fields: Dict[str, str] = {
#         'id': 'id',
#         'title': 'title.raw',
#     }

#     ordering: tuple = ('_score', 'id',)



from typing import Any, Callable, cast

import structlog
from django.contrib.auth.models import AnonymousUser
from django.http import HttpRequest, HttpResponse

from authorization.middleware_utils import (  # Import from middleware.py
    get_user_from_keycloak_token,
)
from authorization.models import User

logger = structlog.getLogger(__name__)


class KeycloakAuthenticationMiddleware:
    """
    Middleware to authenticate users with Keycloak tokens.
    """

    def __init__(self, get_response: Callable[[HttpRequest], HttpResponse]) -> None:
        self.get_response = get_response
        logger.info("KeycloakAuthenticationMiddleware initialized")

    def __call__(self, request: HttpRequest) -> HttpResponse:
        # Skip authentication for OPTIONS requests
        if request.method == "OPTIONS":
            logger.debug("Skipping authentication for OPTIONS request")
            return self.get_response(request)

        # Process the request before the view is called
        if not hasattr(request, "user") or request.user.is_anonymous:
            logger.debug("Setting user from Keycloak token")
            # Set user directly instead of using SimpleLazyObject to avoid potential issues
            try:
                request.user = get_user_from_keycloak_token(request)
                logger.debug(
                    f"User set: {request.user}, authenticated: {request.user.is_authenticated}"
                )
            except Exception as e:
                logger.error(f"Error setting user: {str(e)}")
                request.user = AnonymousUser()
        else:
            logger.debug(
                f"User already set: {request.user}, authenticated: {request.user.is_authenticated}"
            )

        # Call the next middleware or view
        response = self.get_response(request)

        return response



import re
from typing import Any, Dict

from django.core.exceptions import ValidationError


def regex_validator(value: Any, pattern: str) -> None:
    if not re.match(pattern, value):
        raise ValidationError(f"Value '{value}' does not match the required pattern.")


def min_length_validator(value: Any, min_length: int) -> None:
    if len(value) < min_length:
        raise ValidationError(
            f"Value '{value}' must be at least {min_length} characters long."
        )


def validate_metadata_value(value: Any) -> bool:
    """
    Validate if the metadata value is of a supported type
    """
    supported_types = (str, int, float, bool, list, dict)
    return isinstance(value, supported_types)


def validate_metadata_dict(metadata: Dict[str, Any]) -> bool:
    """
    Validate if all values in the metadata dictionary are of supported types
    """
    return all(validate_metadata_value(value) for value in metadata.values())


VALIDATOR_MAP = {
    "regex_validator": regex_validator,
    "min_length_validator": min_length_validator,
}



import uuid

from django.db import models

from api.utils.enums import ChartStatus

# Import the Dataset model indirectly to avoid duplicate import error
from api.utils.file_paths import _chart_image_directory_path


class ResourceChartImage(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=50, unique=False, blank=True)
    description = models.CharField(max_length=1000, unique=False, blank=True)
    image = models.ImageField(
        upload_to=_chart_image_directory_path, blank=True, null=True, max_length=300
    )
    dataset = models.ForeignKey(
        "api.Dataset",
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        related_name="chart_images",
    )
    modified = models.DateTimeField(auto_now=True)
    status = models.CharField(
        max_length=50,
        choices=ChartStatus.choices,
        default=ChartStatus.DRAFT,
        blank=False,
        unique=False,
    )

    def __str__(self) -> str:
        return self.name

    class Meta:
        db_table = "resource_chart_image"



import uuid
from typing import Any, List, Optional, TypeVar

import strawberry
import structlog
from django.db.models import QuerySet
from strawberry import auto
from strawberry_django import type

from api.models import (
    Resource,
    ResourceFileDetails,
    ResourceMetadata,
    ResourcePreviewDetails,
    ResourceSchema,
)
from api.types.base_type import BaseType
from api.types.type_file_details import TypeFileDetails
from api.types.type_preview_data import PreviewData
from api.types.type_resource_metadata import TypeResourceMetadata
from api.utils.data_indexing import get_preview_data, get_row_count
from api.utils.graphql_telemetry import trace_resolver

logger = structlog.get_logger(__name__)

T = TypeVar("T", bound="TypeResource")


@type(ResourceSchema, fields="__all__")
class TypeResourceSchema(BaseType):
    """Type for resource schema."""

    id: auto
    resource: auto
    field_name: auto
    format: auto
    description: auto
    created: auto
    modified: auto


@type(ResourcePreviewDetails, fields="__all__")
class TypePreviewDetails(BaseType):
    """Type for preview details."""

    pass


@type(Resource)
class TypeResource(BaseType):
    """Type for resource."""

    id: uuid.UUID
    dataset: auto
    created: auto
    modified: auto
    type: auto
    name: auto
    description: auto
    preview_enabled: auto
    preview_details: Optional[TypePreviewDetails]
    download_count: auto

    # @strawberry.field
    # def model_resources(self) -> List[TypeAccessModelResourceFields]:
    #     """Get access model resources for this resource.

    #     Returns:
    #         List[TypeAccessModelResourceFields]: List of access model resources
    #     """
    #     try:
    #         queryset = AccessModelResource.objects.filter(resource_id=self.id)
    #         return TypeAccessModelResourceFields.from_django_list(queryset)
    #     except (AttributeError, AccessModelResource.DoesNotExist):
    #         return []

    @strawberry.field
    @trace_resolver(name="get_resource_metadata", attributes={"component": "resource"})
    def metadata(self) -> List[TypeResourceMetadata]:
        """Get metadata for this resource
        Returns:
            List[TypeResourceMetadata]: List of resource metadata
        """
        try:
            queryset: QuerySet = ResourceMetadata.objects.filter(resource__id=self.id)
            return TypeResourceMetadata.from_django_list(queryset)
        except (AttributeError, ResourceMetadata.DoesNotExist):
            return []

    # @strawberry.field
    # def access_models(self) -> List[TypeAccessModel]:
    #     """Get access models for this resource.

    #     Returns:
    #         List[TypeAccessModel]: List of access models
    #     """
    #     try:
    #         model_resources = AccessModelResource.objects.filter(resource_id=self.id)
    #         queryset: QuerySet[AccessModel] = AccessModel.objects.filter(
    #             id__in=[mr.access_model.id for mr in model_resources]  # type: ignore
    #         )
    #         return TypeAccessModel.from_django_list(queryset)
    #     except (AttributeError, AccessModel.DoesNotExist):
    #         return []

    @strawberry.field
    @trace_resolver(
        name="get_resource_file_details", attributes={"component": "resource"}
    )
    def file_details(self) -> Optional[TypeFileDetails]:
        """Get file details for this resource.

        Returns:
            Optional[TypeFileDetails]: File details if they exist, None otherwise
        """
        try:
            details = getattr(self, "resourcefiledetails", None)
            if details is None:
                return None
            return TypeFileDetails.from_django(details)
        except (AttributeError, ResourceFileDetails.DoesNotExist):
            return None

    @strawberry.field
    @trace_resolver(name="get_resource_schema", attributes={"component": "resource"})
    def schema(self) -> List[TypeResourceSchema]:
        """Get schema for this resource.

        Returns:
            List[TypeResourceSchema]: List of resource schema
        """
        try:
            queryset = getattr(self, "resourceschema_set", None)
            if queryset is None:
                return []
            return TypeResourceSchema.from_django_list(queryset.all())
        except (AttributeError, ResourceSchema.DoesNotExist):
            return []

    @strawberry.field
    @trace_resolver(
        name="get_resource_preview_data", attributes={"component": "resource"}
    )
    def preview_data(self) -> PreviewData:
        """Get preview data for the resource.

        Returns:
            PreviewData: Preview data with columns and rows if successful, or an empty PreviewData object if not available
        """
        try:
            file_details = getattr(self, "resourcefiledetails", None)
            if not file_details or not getattr(self, "preview_details", None):
                return PreviewData(columns=[], rows=[])
            if not getattr(
                self, "preview_enabled", False
            ) or not file_details.format.lower() in ["csv", "xls", "xlsx"]:
                return PreviewData(columns=[], rows=[])

            try:
                result = get_preview_data(self)  # type: ignore
                if result is None:
                    return PreviewData(columns=[], rows=[])
                return result
            except Exception as preview_error:
                logger.error(f"Error in get_preview_data: {str(preview_error)}")
                return PreviewData(columns=[], rows=[])
        except Exception as e:
            logger.error(f"Error loading preview data: {str(e)}")
            return PreviewData(columns=[], rows=[])

    @strawberry.field
    @trace_resolver(
        name="get_resource_no_of_entries", attributes={"component": "resource"}
    )
    def no_of_entries(self) -> int:
        """Get the number of entries in the resource."""
        try:
            file_details = getattr(self, "resourcefiledetails", None)
            if not file_details:
                return 0

            if (
                not hasattr(file_details, "format")
                or file_details.format.lower() != "csv"
            ):
                return 0

            try:
                return get_row_count(self)  # type: ignore
            except Exception as row_count_error:
                logger.error(f"Error in get_row_count: {str(row_count_error)}")
                return 0
        except Exception as e:
            logger.error(f"Error getting number of entries: {str(e)}")
            return 0



"""
API Client for making requests to AI model endpoints.
Supports various authentication methods and providers.
"""

import json
import time
from typing import Any, Dict, Optional

import httpx
from asgiref.sync import sync_to_async
from django.utils import timezone
from tenacity import retry, stop_after_attempt, wait_exponential  # type: ignore

from api.models import AIModel, ModelAPIKey, ModelEndpoint


class ModelAPIClient:
    """Client for interacting with AI model APIs"""

    def __init__(self, model: AIModel):
        self.model = model
        self.endpoint = model.get_primary_endpoint()
        if not self.endpoint:
            raise ValueError(f"No primary endpoint configured for model {model.name}")

        # Get API key if available
        self.api_key = self._get_api_key()

    def _get_api_key(self) -> Optional[str]:
        """Get the active API key for this model"""
        api_key_obj = self.model.api_keys.filter(is_active=True).first()
        if api_key_obj:
            return api_key_obj.get_key()
        return None

    def _build_headers(self) -> Dict[str, str]:
        """Build request headers including authentication"""
        headers = {"Content-Type": "application/json", **self.endpoint.headers}

        # Add authentication header
        if self.api_key and self.endpoint.auth_type != "NONE":
            if self.endpoint.auth_type == "BEARER":
                headers[self.endpoint.auth_header_name] = f"Bearer {self.api_key}"
            elif self.endpoint.auth_type == "API_KEY":
                headers[self.endpoint.auth_header_name] = self.api_key
            elif self.endpoint.auth_type == "CUSTOM":
                # Custom headers should be in endpoint.headers
                pass

        return headers

    def _build_request_body(
        self, input_text: str, parameters: Optional[Dict[str, Any]] = None
    ) -> Dict[str, Any]:
        """Build request body based on provider and template"""
        body: Dict[str, Any]
        if self.endpoint.request_template:
            # Use custom template
            template_copy = self.endpoint.request_template.copy()
            # Replace placeholders
            body = self._replace_placeholders(template_copy, input_text, parameters or {})
        else:
            # Default templates based on provider
            body = self._get_default_template(input_text, parameters or {})

        return body

    def _replace_placeholders(self, template: Dict, input_text: str, parameters: Dict) -> Dict:
        """Replace placeholders in template"""
        result: Dict[str, Any] = {}
        for key, value in template.items():
            if isinstance(value, str):
                replaced_value = value.replace("{input}", input_text)
                replaced_value = replaced_value.replace("{prompt}", input_text)
                # Replace parameter placeholders
                for param_key, param_value in parameters.items():
                    replaced_value = replaced_value.replace(f"{{{param_key}}}", str(param_value))
                result[key] = replaced_value
            elif isinstance(value, dict):
                result[key] = self._replace_placeholders(value, input_text, parameters)
            elif isinstance(value, list):
                result[key] = [
                    (
                        self._replace_placeholders(item, input_text, parameters)
                        if isinstance(item, dict)
                        else item
                    )
                    for item in value
                ]
            else:
                result[key] = value
        return result

    def _get_default_template(self, input_text: str, parameters: Dict) -> Dict[str, Any]:
        """Get default request template based on provider"""
        provider = self.model.provider.upper()

        if provider == "OPENAI":
            return {
                "model": self.model.provider_model_id or "gpt-3.5-turbo",
                "messages": [{"role": "user", "content": input_text}],
                "temperature": parameters.get("temperature", 0.7),
                "max_tokens": parameters.get("max_tokens", self.model.max_tokens or 1000),
            }
        elif "LLAMA" in provider:
            # Llama models - format depends on provider
            if "OLLAMA" in provider:
                return {
                    "model": self.model.provider_model_id or "llama2",
                    "prompt": input_text,
                    "stream": False,
                    "options": {
                        "temperature": parameters.get("temperature", 0.7),
                        "num_predict": parameters.get("max_tokens", self.model.max_tokens or 1000),
                    },
                }
            elif "TOGETHER" in provider:
                return {
                    "model": self.model.provider_model_id or "togethercomputer/llama-2-7b-chat",
                    "prompt": input_text,
                    "temperature": parameters.get("temperature", 0.7),
                    "max_tokens": parameters.get("max_tokens", self.model.max_tokens or 1000),
                }
            elif "REPLICATE" in provider:
                return {
                    "version": self.model.provider_model_id,
                    "input": {
                        "prompt": input_text,
                        "temperature": parameters.get("temperature", 0.7),
                        "max_length": parameters.get("max_tokens", self.model.max_tokens or 1000),
                    },
                }
            else:
                # Generic Llama format (OpenAI-compatible)
                return {
                    "model": self.model.provider_model_id or "llama-2-7b-chat",
                    "messages": [{"role": "user", "content": input_text}],
                    "temperature": parameters.get("temperature", 0.7),
                    "max_tokens": parameters.get("max_tokens", self.model.max_tokens or 1000),
                }
        else:
            # Generic template for custom APIs
            return {"input": input_text, "parameters": parameters}

    def _extract_response(self, response_data: Dict) -> str:
        """Extract text response from API response"""
        if self.endpoint.response_path:
            # Use custom response path
            result: Any = self._get_nested_value(response_data, self.endpoint.response_path)
            return str(result)

        # Default extraction based on provider
        provider = self.model.provider.upper()

        try:
            if provider == "OPENAI":
                return str(response_data["choices"][0]["message"]["content"])
            elif "LLAMA" in provider:
                if "OLLAMA" in provider:
                    return str(response_data["response"])
                elif "TOGETHER" in provider:
                    return str(response_data["output"]["choices"][0]["text"])
                elif "REPLICATE" in provider:
                    # Replicate returns array of strings
                    output = response_data.get("output", [])
                    return "".join(output) if isinstance(output, list) else str(output)
                else:
                    # Try OpenAI-compatible format first
                    if "choices" in response_data:
                        return str(response_data["choices"][0]["message"]["content"])
                    elif "response" in response_data:
                        return str(response_data["response"])
                    elif "text" in response_data:
                        return str(response_data["text"])

            # Try common patterns for custom APIs
            if "text" in response_data:
                return str(response_data["text"])
            elif "output" in response_data:
                return str(response_data["output"])
            elif "response" in response_data:
                return str(response_data["response"])
            elif "content" in response_data:
                return str(response_data["content"])
            else:
                return str(response_data)
        except (KeyError, IndexError, TypeError) as e:
            raise ValueError(f"Could not extract response from API: {e}")

    def _get_nested_value(self, data: Dict, path: str) -> Any:
        """Get nested value from dict using dot notation (e.g., 'choices[0].message.content')"""
        import re

        keys = re.split(r"\.|\[|\]", path)
        keys = [k for k in keys if k]  # Remove empty strings

        current = data
        for key in keys:
            if key.isdigit():
                current = current[int(key)]
            else:
                current = current[key]

        return current

    async def _update_endpoint_success(self) -> None:
        """Update endpoint statistics on successful call (async-safe)"""

        def _update() -> None:
            self.endpoint.total_requests += 1
            self.endpoint.last_success_at = timezone.now()
            self.endpoint.save(update_fields=["total_requests", "last_success_at"])

        await sync_to_async(_update)()

    async def _update_endpoint_failure(self) -> None:
        """Update endpoint statistics on failed call (async-safe)"""

        def _update() -> None:
            self.endpoint.failed_requests += 1
            self.endpoint.last_failure_at = timezone.now()
            self.endpoint.save(update_fields=["failed_requests", "last_failure_at"])

        await sync_to_async(_update)()

    @retry(stop=stop_after_attempt(3), wait=wait_exponential(multiplier=1, min=2, max=10))
    async def call_async(
        self, input_text: str, parameters: Optional[Dict[str, Any]] = None
    ) -> Dict[str, Any]:
        """Make async API call to the model"""
        start_time = time.time()

        headers = self._build_headers()
        body = self._build_request_body(input_text, parameters)

        try:
            async with httpx.AsyncClient(timeout=self.endpoint.timeout_seconds) as client:
                if self.endpoint.http_method == "POST":
                    response = await client.post(self.endpoint.url, headers=headers, json=body)
                elif self.endpoint.http_method == "GET":
                    response = await client.get(self.endpoint.url, headers=headers, params=body)
                else:
                    raise ValueError(f"Unsupported HTTP method: {self.endpoint.http_method}")

                response.raise_for_status()
                response_data = response.json()

                latency_ms = (time.time() - start_time) * 1000

                # Update endpoint statistics (async-safe)
                await self._update_endpoint_success()

                # Extract response text
                output_text = self._extract_response(response_data)

                return {
                    "success": True,
                    "output": output_text,
                    "raw_response": response_data,
                    "latency_ms": latency_ms,
                    "status_code": response.status_code,
                }

        except httpx.HTTPStatusError as e:
            # Update failure statistics (async-safe)
            await self._update_endpoint_failure()

            return {
                "success": False,
                "error": f"HTTP {e.response.status_code}: {e.response.text}",
                "status_code": e.response.status_code,
                "latency_ms": (time.time() - start_time) * 1000,
            }
        except Exception as e:
            # Update failure statistics (async-safe)
            await self._update_endpoint_failure()

            return {
                "success": False,
                "error": str(e),
                "latency_ms": (time.time() - start_time) * 1000,
            }

    def call(self, input_text: str, parameters: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        """Make synchronous API call to the model"""
        import asyncio

        # Run async call in sync context
        try:
            loop = asyncio.get_event_loop()
            if loop.is_running():
                # If loop is already running, use nest_asyncio
                import nest_asyncio  # type: ignore

                nest_asyncio.apply()
                return loop.run_until_complete(self.call_async(input_text, parameters))
            else:
                return asyncio.run(self.call_async(input_text, parameters))
        except RuntimeError:
            # No event loop, create one
            return asyncio.run(self.call_async(input_text, parameters))



from typing import Any, Dict, Optional, Type, TypeVar, Union

from actstream.actions import action
from django.conf import settings
from django.contrib.auth.models import AbstractBaseUser
from django.contrib.contenttypes.models import ContentType
from django.db.models import Model
from django.http import HttpRequest

# mypy: disable-error-code=no-any-return


def record_activity(
    actor: Union[AbstractBaseUser, Model],
    verb: str,
    action_object: Optional[Model] = None,
    target: Optional[Model] = None,
    request: Optional[HttpRequest] = None,
    **kwargs: Any
) -> Optional[Model]:
    """
    Record an activity based on settings and user consent.
    This function wraps the actstream.actions.action function and adds consent checking.

    Args:
        actor: The actor performing the activity (usually a user)
        verb: The verb describing the activity (e.g., 'created', 'updated')
        action_object: The object being acted upon
        target: The target of the activity
        request: The current request object (used to check consent)
        **kwargs: Additional arguments to pass to the action function

    Returns:
        The created Action instance or None if consent was not given
    """
    # Get consent settings
    require_consent = getattr(settings, "ACTIVITY_CONSENT", {}).get(
        "REQUIRE_CONSENT", True
    )
    track_anonymous = getattr(settings, "ACTIVITY_CONSENT", {}).get(
        "TRACK_ANONYMOUS", False
    )

    # If consent is not required, we can record the activity without checking consent
    if not require_consent:
        # For anonymous users (when actor is not a user model), check track_anonymous setting
        if not hasattr(actor, "is_authenticated"):
            if not track_anonymous:
                return None
        return action(
            actor=actor, verb=verb, action_object=action_object, target=target, **kwargs
        )

    # If consent is required, check if we have consent
    # Check if we have a request and if the user has given consent
    if request and hasattr(request, "has_activity_consent"):
        if not request.has_activity_consent:
            return None

    # If we don't have a request, we need to check the actor's consent directly
    # This is useful for background tasks where there is no request
    elif hasattr(actor, "activity_consent"):
        try:
            if not actor.activity_consent.activity_tracking_enabled:
                return None
        except Exception:
            # If there's any error checking consent, don't record the activity
            return None
    # If we can't determine consent, default to not recording the activity when consent is required
    else:
        return None

    # Record the activity
    return action(
        actor=actor, verb=verb, action_object=action_object, target=target, **kwargs
    )



"""Schema definitions for metadata."""

from typing import List, Optional

import strawberry
import strawberry_django
from strawberry.types import Info

from api.models import Metadata
from api.types.type_metadata import TypeMetadata
from api.utils.graphql_telemetry import trace_resolver
from authorization.permissions import IsAuthenticated


@strawberry_django.input(Metadata, fields="__all__")
class MetadataInput:
    """Input type for metadata creation."""

    pass


@strawberry_django.partial(Metadata, fields="__all__")
class MetadataInputPartial:
    """Input type for metadata updates."""

    id: str


@strawberry.type(name="Query")
class Query:
    """Queries for metadata."""

    @strawberry_django.field(pagination=True)
    def metadata_list(
        self,
        info: Info,
        first: Optional[int] = None,
        after: Optional[str] = None,
    ) -> List[TypeMetadata]:
        """Get paginated metadata list."""
        metadata_list = Metadata.objects.all()
        if first is not None:
            metadata_list = metadata_list[:first]
        if after is not None:
            metadata_list = metadata_list.filter(id__gt=after)
        return [TypeMetadata.from_django(meta) for meta in metadata_list]


@strawberry.type
class Mutation:
    """Mutations for metadata."""

    create_metadata: TypeMetadata = strawberry_django.mutations.create(MetadataInput)
    update_metadata: TypeMetadata = strawberry_django.mutations.update(
        MetadataInputPartial
    )

    @strawberry_django.mutation(
        handle_django_errors=False,
        permission_classes=[IsAuthenticated],
    )
    @trace_resolver(
        name="delete_metadata",
        attributes={"component": "metadata", "operation": "mutation"},
    )
    def delete_metadata(self, info: Info, metadata_id: str) -> bool:
        try:
            metadata = Metadata.objects.get(id=metadata_id)
            metadata.delete()
            return True
        except Metadata.DoesNotExist as e:
            raise ValueError(f"Metadata with ID {metadata_id} does not exist.")



from typing import List

import strawberry
from strawberry.scalars import JSON


@strawberry.type
class PreviewData:
    """Type for preview data."""

    columns: List[str]
    rows: List[List[JSON]]



from datetime import datetime
from typing import (
    Any,
    Dict,
    Generic,
    List,
    Optional,
    Sequence,
    Type,
    TypeVar,
    Union,
    cast,
)

import strawberry
from django.db.models import Model, QuerySet
from strawberry_django.fields.field import StrawberryDjangoField

T = TypeVar("T", bound="BaseType")
M = TypeVar("M", bound=Model)


class BaseType:
    """Base class for all GraphQL types with helper methods for Django model conversion."""

    @classmethod
    def from_django(cls: Type[T], instance: M) -> T:
        """Convert a Django model instance to a Strawberry type."""
        if not instance:
            raise ValueError(f"Cannot convert None to {cls.__name__}")

        data: Dict[str, Any] = {
            field.name: getattr(instance, field.name) for field in instance._meta.fields
        }

        try:
            return cast(T, instance)  # Explicitly cast to the correct type
        except TypeError as e:
            raise ValueError(f"Failed to create {cls.__name__} from data: {data}")

    @classmethod
    def from_dict(cls: Type[T], data: Dict[str, Any]) -> Optional[T]:
        """Create an instance from a dictionary."""
        if not data:
            return None

        try:
            return cls(**data)  # Explicitly instantiate the class
        except (KeyError, TypeError, ValueError):
            return None

    @classmethod
    def from_django_list(
        cls: Type[T], instances: Union[Sequence[M], QuerySet[M]]
    ) -> List[T]:
        """Convert a list or QuerySet of Django model instances to Strawberry types."""
        # return [cls.from_django(instance) for instance in instances]
        return cast(List[T], instances)



import logging
from typing import Any, List, Optional, cast

from django.core.management.base import BaseCommand, CommandParser

from authorization.models import User

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Promotes a user to superuser status by username or email"  # type: ignore[assignment]

    def add_arguments(self, parser: CommandParser) -> None:  # type: ignore[no-untyped-def]
        parser.add_argument(
            "identifier",
            type=str,
            help="Username or email of the user to promote",
        )

    def handle(self, *args: Any, **options: Any) -> None:  # type: ignore[no-untyped-def]
        identifier = options["identifier"]

        # Try to find the user by username, email, or keycloak_id
        user = None
        try:
            # Try username first
            user = User.objects.filter(username=identifier).first()
            if user:
                self.stdout.write(
                    self.style.SUCCESS(f"Found user by username: {user.username}")
                )

            # Try email next
            if not user:
                user = User.objects.filter(email=identifier).first()
                if user:
                    self.stdout.write(
                        self.style.SUCCESS(f"Found user by email: {user.email}")
                    )

            # Try keycloak_id last
            if not user:
                user = User.objects.filter(keycloak_id=identifier).first()
                if user:
                    self.stdout.write(
                        self.style.SUCCESS(
                            f"Found user by keycloak_id: {user.keycloak_id}"
                        )
                    )

            if not user:
                self.stdout.write(
                    self.style.ERROR(f"User with identifier '{identifier}' not found")
                )
                self.stdout.write(
                    self.style.ERROR(
                        "Make sure the user has logged in at least once or check the identifier"
                    )
                )
                return

            # Promote the user to superuser
            user.is_staff = True
            user.is_superuser = True
            user.save()

            self.stdout.write(
                self.style.SUCCESS(
                    f"Successfully promoted {user.username} (ID: {user.id}) to superuser status"
                )
            )
        except Exception as e:
            self.stdout.write(self.style.ERROR(f"Error promoting user: {str(e)}"))



import strawberry_django

from api.models import CollaborativeMetadata
from api.types.base_type import BaseType
from api.types.type_metadata import TypeMetadata


@strawberry_django.type(CollaborativeMetadata, fields="__all__")
class TypeCollaborativeMetadata(BaseType):
    metadata_item: TypeMetadata



FORMAT_MAPPING = {
    "text/csv": "CSV",
    "csv": "CSV",
    "application/csv": "CSV",
    "application/json": "JSON",
    "json": "JSON",
    # "application/msword": "DOC",
    # "application/doc": "DOC",
    # "application/ms-doc": "DOC",
    "application/pdf": "PDF",
    # "application/vnd.openxmlformats-officedocument.wordprocessingml.document": "DOCX",
    "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet": "XLSX",
    "application/vnd.ms-excel": "XLS",
    "application/xml": "XML",
    "xml": "XML",
    "text/xml": "XML",
    "application/geo+json": "JSON",
    "application/gml+xml": "XML",
    "application/xhtml+xml": "XML",
    # TODO: handle csv identified as text/plain
    "text/plain": "CSV",
    "application/zip": "ZIP",
}

IMAGE_FORMAT_MAPPING = {
    "image/png": "PNG",
    "image/jpeg": "JPEG",
    "image/svg+xml": "SVG",
}
